*========================== PROGRAM: PSI_ALL_MEASURES.SAS ===========================;
*
*  DESCRIPTION:
*         Assigns the patient safety outcome measure of interest, stratifiers, and
*         comorbidity flags to inpatient records.
*         Variables created by this program are TPPSxx,QPPSxx,PPPSxx,TAPSxx,
*         stratifiers, comorbidities, and some variables used for risk adjustment.
*
*                     >>> AHRQ QI ICD-09  VERSION 6.0.2 <<<
*                        >>> VERSION DATE JULY 2017 <<<
*
*  USER NOTE1: Make sure you have created the format library using
*              PSI_ALL_FORMATS.SAS BEFORE running this program.
*
*  USER NOTE2: The AHRQ QI software does not support the calculation of weighted 
*              estimates and standard errors using complex sampling designs. 
*
*  USER NOTE3: See the PSI_ICD09_v6_Release_Notes.txt file for software change notes.
*
*  USER NOTE4: Area-level PSI measures are being retired and will not appear in v7.0.
*====================================================================================;
*PATHNAME specifies the location of the PSI folder which includes the
          Programs, Macros, and SASdata subfolders;
%let PATHNAME=C:\PATHNAME\PSI;                           *<===USER MUST modify; 
filename CONTROL "&PATHNAME.\Programs\PSI_ALL_CONTROL.SAS"; 
%include CONTROL;

 title2 'PSI_ALL_MEASURES';
 title3 'AHRQ PATIENT SAFETY INDICATORS: ASSIGN QIS TO INPATIENT DATA';

 * ------------------------------------------------------------------- ;
 * --- CREATE A PERMANENT DATASET CONTAINING ALL RECORDS THAT WILL --- ; 
 * --- NOT BE INCLUDED IN ANALYSIS BECAUSE OF AGE OR KEY VARIABLE  --- ;
 * --- VALUES ARE MISSING. REVIEW AFTER RUNNING PSI_ALL_MEASURES.  --- ;
 * ------------------------------------------------------------------- ;
 
 data OUTMSR.&DELFILE.
        (keep=KEY HOSPID SEX AGE DX1 MDC YEAR DQTR);
   set INMSR.&DISCHARGE.;
   if (AGE lt 0) or  (AGE lt 18 and MDC notin (14)) or (SEX le 0) or 
       (compress(DX1) in ('')) or (DQTR le .Z) or (YEAR le .Z);
 run;

 * -------------------------------------------------------------- ;
 * --- PATIENT SAFETY INDICATOR (PSI) NAMING CONVENTION:      --- ;
 * --- THE FIRST LETTER IDENTIFIES THE PATIENT SAFETY         --- ;
 * --- INDICATOR AS ONE OF THE FOLLOWING:
                (T) NUMERATOR ("TOP")
                (P) POPULATION ("POP")
                (Q) EXCLUSION FLAG BASED ON POA
 * --- THE SECOND LETTER IDENTIFIES THE PSI AS A PROVIDER (P) --- ;
 * --- OR AN AREA (A) LEVEL INDICATOR.  THE                   --- ;
 * --- NEXT TWO CHARACTERS ARE ALWAYS 'PS'. THE LAST TWO      --- ;
 * --- DIGITS ARE THE INDICATOR NUMBER (WITHIN THAT SUBTYPE). --- ;
 * -------------------------------------------------------------- ;

 *--- Load standard diagnosis and procedure macros ---* ;
 %include MacLib (PSI_Dx_Px_Macros.sas);

 data OUTMSR.&OUTFILE_MEAS. 
    (keep=KEY HOSPID FIPST FIPSTCO DRG DRGVER MDC MDRG YEAR DQTR 
          AGECAT POPCAT SEXCAT RACECAT PAYCAT DUALCAT SEX AGE
          LOS TRNSFER  

          CHF VALVE PULMCIRC PERIVASC HTN_C
          PARA NEURO CHRNLUNG DM DMCX
          HYPOTHY RENLFAIL LIVER ULCER AIDS
          LYMPH METS TUMOR ARTH OBESE
          WGHTLOSS BLDLOSS ANEMDEF ALCOHOL DRUG
          PSYCH DEPRESS COAG LYTES IMMUNE

          TPPS02-TPPS03  TPPS05-TPPS16 TPPS18 TPPS19
          QPPS03 QPPS05-QPPS16 TAPS21-TAPS27
          PPPS02-PPPS03  PPPS05-PPPS16 PPPS18 PPPS19 

          TPPS04 TPPS04A--TPPS04E  
          XPPS04A_ANY--XPPS04E_ANY
          XPPS04A_SEVERE--XPPS04E_SEVERE
          PPPS04  PPPS04A--PPPS04E);

 set   INMSR.&DISCHARGE. 
      (keep=KEY HOSPID DRG DRGVER MDC SEX AGE PSTCO
            RACE YEAR DQTR PAY1 PAY2
            DISP LOS ASOURCE POINTOFORIGINUB04 ATYPE
            DX1-DX&NDX. PR1-PR&NPR. %ADDPRDAY 
            DXPOA1-DXPOA&NDX.);

 * -------------------------------------------------------------- ;
 * --- DEFINE MDC                        ------------------------ ;
 * -------------------------------------------------------------- ;

 attrib MDCNEW length=3
 label='IMPUTED MDC';

 if DRGVER = . then do;
    if      (YEAR in (2007) and DQTR in (4))     then DRGVER = 25;
    else if (YEAR in (2008) and DQTR in (1,2,3)) then DRGVER = 25;
    else if (YEAR in (2008) and DQTR in (4))     then DRGVER = 26;
    else if (YEAR in (2009) and DQTR in (1,2,3)) then DRGVER = 26;
    else if (YEAR in (2009) and DQTR in (4))     then DRGVER = 27;
    else if (YEAR in (2010) and DQTR in (1,2,3)) then DRGVER = 27;
    else if (YEAR in (2010) and DQTR in (4))     then DRGVER = 28;
    else if (YEAR in (2011) and DQTR in (1,2,3)) then DRGVER = 28;
    else if (YEAR in (2011) and DQTR in (4))     then DRGVER = 29;
    else if (YEAR in (2012) and DQTR in (1,2,3)) then DRGVER = 29;
    else if (YEAR in (2012) and DQTR in (4))     then DRGVER = 30;
    else if (YEAR in (2013) and DQTR in (1,2,3)) then DRGVER = 30;
    else if (YEAR in (2013) and DQTR in (4))     then DRGVER = 31;
    else if (YEAR in (2014) and DQTR in (1,2,3)) then DRGVER = 31;
    else if (YEAR in (2014) and DQTR in (4))     then DRGVER = 32;
    else if (YEAR in (2015) and DQTR in (1,2,3)) then DRGVER = 32;
 end;

 *------------------- IMPUTE MDC 24 ---------------------* ;
 *-- Define arrays to be used in defining MDC24 below. --* ;
 *-- Eight different body sites are defined:           --* ;
 *-------------------------------------------------------* ;
 *--    1: Head                                        --* ;
 *--    2: Chest                                       --* ;
 *--    3: Abdominal                                   --* ;
 *--    4: Kidney                                      --* ;
 *--    5: Urinary                                     --* ;
 *--    6: Pelvis/spine                                --* ;
 *--    7: Upper limb                                  --* ;
 *--    8: Lower limb                                  --* ; 
 *-------------------------------------------------------* ;

 array site_dx(&ndx.) $ site_dx1-site_dx&ndx.;

 %defsite;

 if put(dx1,$pr.)='1' then trauma_dx1=1; else trauma_dx1=0;


 IF (1<=DRG<=8 OR 10 <=DRG <=14 OR DRG IN (16,17) OR 981<=DRG<=989) then do;
    
 *-- The duplicated DX1 are all mapped to 12 in the format --* ;
 *-- $DX1MDC, so correct them here based on SEX.           --* ;
    
    if DX1 IN ("05410", "05419", "0910",  "0980",  "09810", "09819", "0982",  "09839", "0990", "0991",
               "0992 ", "09940", "09941", "09949", "09950", "09953", "09955", "09959", "0998", "0999",
               "1122",  "13100", "13102", "13109", "1953",  "19882", "4565",  "6981",  "7527", "75289",
               "7529",  "7586",  "75881", "75889", "8676",  "8677",  "8678",  "8679",  "8788", "8789",
               "9082",  "9224",  "9260",  "V252",  "V260",  "V4577") then do;
               if sex=1 then MDC=12;
          else if sex=2 then MDC=13;
    end;
     else MDC=PUT(DX1,$DX1MDC.);

 *-- MDC 24 requires the review of multiple diagnoses --*;

     if (981 <=DRG <= 989) then do;
    Flag_TwoSitesInDXs=0;
    do i = 2 to &ndx.;
       do j = (i+1) to &ndx.;
          if site_dx(i)^=' ' and site_dx(j)^=' ' and  site_dx(i)^=site_dx(j) then Flag_TwoSitesInDXs=Flag_TwoSitesInDXs+1;
       end;
    end;

     if trauma_dx1=1 then do;
       if (%mdx1($s1fmt.) and (%mdx2($s2fmt.) or %mdx2($s3fmt.) or %mdx2($s4fmt.) or %mdx2($s5fmt.) or %mdx2($s6fmt.) or %mdx2($s7fmt.) or %mdx2($s8fmt.))) or 
         (%mdx1($s2fmt.) and (%mdx2($s1fmt.) or %mdx2($s3fmt.) or %mdx2($s4fmt.) or %mdx2($s5fmt.) or %mdx2($s6fmt.) or %mdx2($s7fmt.) or %mdx2($s8fmt.))) or 
         (%mdx1($s3fmt.) and (%mdx2($s1fmt.) or %mdx2($s2fmt.) or %mdx2($s4fmt.) or %mdx2($s5fmt.) or %mdx2($s6fmt.) or %mdx2($s7fmt.) or %mdx2($s8fmt.))) or 
         (%mdx1($s4fmt.) and (%mdx2($s1fmt.) or %mdx2($s2fmt.) or %mdx2($s3fmt.) or %mdx2($s5fmt.) or %mdx2($s6fmt.) or %mdx2($s7fmt.) or %mdx2($s8fmt.))) or 
         (%mdx1($s5fmt.) and (%mdx2($s1fmt.) or %mdx2($s2fmt.) or %mdx2($s3fmt.) or %mdx2($s4fmt.) or %mdx2($s6fmt.) or %mdx2($s7fmt.) or %mdx2($s8fmt.))) or 
         (%mdx1($s6fmt.) and (%mdx2($s1fmt.) or %mdx2($s2fmt.) or %mdx2($s3fmt.) or %mdx2($s4fmt.) or %mdx2($s5fmt.) or %mdx2($s7fmt.) or %mdx2($s8fmt.))) or 
         (%mdx1($s7fmt.) and (%mdx2($s1fmt.) or %mdx2($s2fmt.) or %mdx2($s3fmt.) or %mdx2($s4fmt.) or %mdx2($s5fmt.) or %mdx2($s6fmt.) or %mdx2($s8fmt.))) or 
         (%mdx1($s8fmt.) and (%mdx2($s1fmt.) or %mdx2($s2fmt.) or %mdx2($s3fmt.) or %mdx2($s4fmt.) or %mdx2($s5fmt.) or %mdx2($s6fmt.) or %mdx2($s7fmt.))) or 
         (put(DX1,$s1fmt.)='0' and 
            put(DX1,$s2fmt.)='0' and 
            put(DX1,$s3fmt.)='0' and 
            put(DX1,$s4fmt.)='0' and 
            put(DX1,$s5fmt.)='0' and 
            put(DX1,$s6fmt.)='0' and 
            put(DX1,$s7fmt.)='0' and 
            put(DX1,$s8fmt.)='0' and Flag_TwoSitesInDXs>0) then MDC=24;
     end;
     end;
 end;

 *-- Apply standard MDC imputation mappings --* ;

 IF MDC notin (01,02,03,04,05,06,07,08,09,10,
               11,12,13,14,15,16,17,18,19,20,
               21,22,23,24,25)
 then do;
    MDCNEW = put(DRG,MDCF2T.);
    if MDCNEW in (01,02,03,04,05,06,07,08,09,10,
                  11,12,13,14,15,16,17,18,19,20,
                  21,22,23,24,25)
    then MDC=MDCNEW;
    else do;
       if DRG in (999) then MDC = 0;
       else put "INVALID MDC KEY: " KEY " MDC " MDC " DRG " DRG DRGVER;
    end;
 end;

 * -------------------------------------------------------------------------- ;
 * --- DELETE RECORDS WITH MISSING VALUES FOR AGE, SEX, DX1, DQTR, OR YEAR -- ;
 * --- DELETE NON ADULT RECORDS --------------------------------------------- ;
 * -------------------------------------------------------------------------- ;
 
 if SEX le 0 then delete;
 if AGE lt 0 then delete;
 if AGE lt 18 and MDC notin (14) then delete;
 if compress(DX1) in ('') then delete;
 if DQTR le .Z then delete;
 if YEAR le .Z then delete;

 * -------------------------------------------------------------- ;
 * --- DEFINE FIPS STATE AND COUNTY CODES ----------------------- ;
 * -------------------------------------------------------------- ;
 
 attrib FIPSTCO length=$5
   label='FIPS STATE COUNTY CODE';
 FIPSTCO = put(PSTCO,Z5.);

 attrib FIPST length=$2
   label='STATE FIPS CODE';
 FIPST = SUBSTR(FIPSTCO,1,2);

 * -------------------------------------------------------------- ;
 * --- DEFINE ICD-9-CM VERSION ---------------------------------- ;
 * -------------------------------------------------------------- ;
 
 attrib ICDVER length=3
   label='ICD-9-CM VERSION';

 ICDVER = 0;
 if      (YEAR in (2007) and DQTR in (4))     then ICDVER = 25;
 else if (YEAR in (2008) and DQTR in (1,2,3)) then ICDVER = 25;
 else if (YEAR in (2008) and DQTR in (4))     then ICDVER = 26;
 else if (YEAR in (2009) and DQTR in (1,2,3)) then ICDVER = 26;
 else if (YEAR in (2009) and DQTR in (4))     then ICDVER = 27;
 else if (YEAR in (2010) and DQTR in (1,2,3)) then ICDVER = 27;
 else if (YEAR in (2010) and DQTR in (4))     then ICDVER = 28;
 else if (YEAR in (2011) and DQTR in (1,2,3)) then ICDVER = 28;
 else if (YEAR in (2011) and DQTR in (4))     then ICDVER = 29;
 else if (YEAR in (2012) and DQTR in (1,2,3)) then ICDVER = 29;
 else if (YEAR in (2012) and DQTR in (4))     then ICDVER = 30;
 else if (YEAR in (2013) and DQTR in (1,2,3)) then ICDVER = 30;
 else if (YEAR in (2013) and DQTR in (4))     then ICDVER = 31;
 else if (YEAR in (2014) and DQTR in (1,2,3)) then ICDVER = 31;
 else if (YEAR in (2014) and DQTR in (4))     then ICDVER = 32;
 else if (YEAR in (2015) and DQTR in (1,2,3)) then ICDVER = 32;

 * ----------------------------- ;
 * --- DEFINE MEDICAL DRGS ----- ;
 * ----------------------------- ;

 attrib MEDICDR length=3
   label='MEDICAL DRGS';

 if %MDR($MEDIC2R.) 
 then MEDICDR = 1;
 else MEDICDR = 0;

 * -------------------------------------------------------------- ;
 * --- DEFINE SURGICAL DRGS ------------------------------------- ;
 * -------------------------------------------------------------- ;
 
 attrib SURGIDR length=3
   label='SURGICAL DRGS';

 if %MDR($SURGI2R.) 
 then SURGIDR = 1;
 else SURGIDR = 0;


 * -------------------------------------------------------------- ;
 * --- DEFINE MODIFIED DRGS ------------------------------------- ;
 * -------------------------------------------------------------- ;
 
 attrib MDRG length=4
   label='MODIFIED DRG';

 MDRG = put(put(DRG,Z3.),$DRGF2T.);


 * -------------------------------------------------------------- ;
 * --- DEFINE LOW MORTALITY DRGS -------------------------------- ;
 * -------------------------------------------------------------- ;
 attrib LOWMODR length=3
   label='LOW MORTALITY DRGS';

 LOWMODR = put(MDRG,LOWMODR.);


 * -------------------------------------------------------------- ;
 * --- DEFINE STRATIFIER: PAYER CATEGORY ------------------------ ;
 * -------------------------------------------------------------- ;
 
 attrib PAYCAT length=3
   label='PATIENT PRIMARY PAYER';

 select (PAY1);
   when (1)  PAYCAT = 1;
   when (2)  PAYCAT = 2;
   when (3)  PAYCAT = 3;
   when (4)  PAYCAT = 4;
   when (5)  PAYCAT = 5;
   otherwise PAYCAT = 6;
 end;                

 attrib DUALCAT length=3
   label='PATIENT DUAL ELIGIBLE';

 if (PAY1 in (1) and PAY2 in (2)) or
    (PAY1 in (2) and PAY2 in (1)) 
 then DUALCAT = 1; else DUALCAT = 0;
    

 * -------------------------------------------------------------- ;
 * --- DEFINE STRATIFIER: RACE CATEGORY ------------------------- ;
 * -------------------------------------------------------------- ;
 
 attrib RACECAT length=3
   label='PATIENT RACE/ETHNICITY';

 select (RACE);
   when (1)  RACECAT = 1;
   when (2)  RACECAT = 2;
   when (3)  RACECAT = 3;
   when (4)  RACECAT = 4;
   when (5)  RACECAT = 5;
   otherwise RACECAT = 6;
 end;                 

 * -------------------------------------------------------------- ;
 * --- DEFINE STRATIFIER: AGE CATEGORY  ------------------------- ;
 * -------------------------------------------------------------- ;
 
 attrib AGECAT length=3
   label='PATIENT AGE';

 select;
   when (      AGE < 18)  AGECAT = 0;
   when (18 <= AGE < 40)  AGECAT = 1;
   when (40 <= AGE < 65)  AGECAT = 2;
   when (65 <= AGE < 75)  AGECAT = 3;
   when (75 <= AGE     )  AGECAT = 4;
   otherwise AGECAT = 0;
  
 end;  

 * -------------------------------------------------------------- ;
 * --- DEFINE STRATIFIER: SEX CATEGORY  ------------------------- ;
 * -------------------------------------------------------------- ;

 attrib SEXCAT length=3
   label='PATIENT SEX';

 select (SEX);
   when (1)  SEXCAT = 1;
   when (2)  SEXCAT = 2;
   otherwise SEXCAT = 0;
 end;  

 * -------------------------------------------------------------- ;
 * --- DEFINE STRATIFIER: POPULATION CATEGORY ------------------- ;
 * -------------------------------------------------------------- ;
 
 attrib POPCAT length=3
   label='PATIENT AGE';

 POPCAT=put(AGE,AGEFMT.);

 * -------------------------------------------------------------- ;
 * --- COUNT OPERATING ROOM PROCEDURES AND IDENTIFY FIRST     --- ;
 * --- OPERATING ROOM PROCEDURE                               --- ;
 * -------------------------------------------------------------- ;
 
 attrib ORCNT length=8
   label='OR PROCEDURE COUNT';

 attrib ORDAY length=8
   label='OR PROCEDURE DAY';

%ORCNT;

 * -------------------------------------------------------------- ;
 * --- DEFINE PROVIDER LEVEL INDICATORS ------------------------- ;
 * -------------------------------------------------------------- ;
 
 %MACRO LBL;

 attrib
 TPPS02 length=8
   label='PSI 02 Death Rate in Low-Mortality Diagnosis Related Groups (DRGs) (Numerator)'
 TPPS03 length=8
   label='PSI 03 Pressure Ulcer Rate (Numerator)'
 TPPS04  length=8
   label='PSI 04 Death Rate among Surgical Inpatients with Serious Treatable Conditions (Numerator)'
 TPPS04A length=8
   label='PSI 04A Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum A: Deep Vein Thrombosis/Pulmonary Embolism (DVT/PE) (Numerator)'
 TPPS04B length=8
   label='PSI 04B 04B Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum B: Pneumonia (Numerator)'
 TPPS04C length=8
   label='PSI 04C Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum C: Sepsis (Numerator)'
 TPPS04D length=8
   label='PSI 04D Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum D: Shock/Cardiac Arrest (Numerator)'
 TPPS04E length=8
   label='PSI 04E Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum E: Gastrointestinal (GI) Hemorrhage/Acute Ulcer (Numerator)'
 TPPS05 length=8
   label='PSI 05 Retained Surgical Item or Unretrieved Device Fragment Count (Numerator)'
 TPPS06 length=8
   label='PSI 06 Iatrogenic Pneumothorax Rate (Numerator)'
 TPPS07 length=8
   label='PSI 07 Central Venous Catheter-Related Blood Stream Infection Rate (Numerator)'
 TPPS08 length=8
   label='PSI 08 In Hospital Fall with Hip Fracture Rate (Numerator)'
 TPPS09 length=8
   label='PSI 09 Perioperative Hemorrhage or Hematoma Rate (Numerator)'
 TPPS10 length=8
   label='PSI 10 Postoperative Acute Kidney Injury Requiring Dialysis Rate (Numerator)'
 TPPS11 length=8
   label='PSI 11 Postoperative Respiratory Failure Rate (Numerator)'
 TPPS12 length=8
   label='PSI 12 Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate (Numerator)'
 TPPS13 length=8
   label='PSI 13 Postoperative Sepsis Rate (Numerator)'
 TPPS14 length=8
   label='PSI 14 Postoperative Wound Dehiscence Rate (Numerator)'
 TPPS15 length=8
   label='PSI 15 Unrecognized Abdominopelvic Accidental Puncture or Laceration Rate (Numerator)'
 TPPS16 length=8
   label='PSI 16 Transfusion Reaction Count (Numerator)'
 TPPS18 length=8
   label='PSI 18 Obstetric Trauma Rate - Vaginal Delivery With Instrument (Numerator)'
 TPPS19 length=8
   label='PSI 19 Obstetric Trauma Rate - Vaginal Delivery Without Instrument (Numerator)'
;

 attrib
 QPPS03 length=3
   label = 'PSI 03 Pressure Ulcer Rate (POA) (1=exclude case based on POA)'
 QPPS05 length=3
   label = 'PSI 05 Retained Surgical Item or Unretrieved Device Fragment Count (POA) (1=exclude case based on POA)'
 QPPS06 length=3
   label = 'PSI 06 Iatrogenic Pneumothorax Rate (POA) (1=exclude case based on POA)'
 QPPS07 length=3
   label = 'PSI 07 Central Venous Catheter-Related Blood Stream Infection Rate (POA) (1=exclude case based on POA)'
 QPPS08 length=3
   label = 'PSI 08 In Hospital Fall with Hip Fracture Rate (POA) (1=exclude case based on POA)'
 QPPS09 length=3
   label = 'PSI 09 Perioperative Hemorrhage or Hematoma Rate (POA) (1=exclude case based on POA)'
 QPPS10 length=3
   label = 'PSI 10 Postoperative Acute Kidney Injury Requiring Dialysis Rate (POA) (1=exclude case based on POA)'
 QPPS11 length=3
   label = 'PSI 11 Postoperative Respiratory Failure Rate (POA) (1=exclude based on POA)'
 QPPS12 length=3
   label = 'PSI 12 Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate (POA) (1=exclude based on POA)'
 QPPS13 length=3
   label = 'PSI 13 Postoperative Sepsis Rate (POA) (1=exclude based on POA)'
 QPPS14 length=3
   label = 'PSI 14 Postoperative Wound Dehiscence Rate (POA) (1=exclude based on POA)'
 QPPS15 length=3
   label = 'PSI 15 Unrecognized Abdominopelvic Accidental Puncture or Laceration Rate (POA) (1=exclude based on POA)'
 QPPS16 length=3
   label = 'PSI 16 Transfusion Reaction Count (POA) (1=exclude based on POA)'
;
 attrib 
 PPPS02 length=3
   label = 'PSI 02 Death Rate in Low-Mortality Diagnosis Related Groups (DRGs) (Pop)'
 PPPS03 length=3
   label = 'PSI 03 Pressure Ulcer Rate (Pop)'
 PPPS04 length=3
   label = 'PSI 04 Death Rate among Surgical Inpatients with Serious Treatable Conditions (Pop)'
 PPPS04A length=3
   label = 'PSI 04A Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum A: Deep Vein Thrombosis/Pulmonary Embolism (DVT/PE) (Pop)'
 PPPS04B length=3
   label = 'PSI 04B Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum B: Pneumonia (Pop)'
 PPPS04C length=3
   label = 'PSI 04C Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum C: Sepsis (Pop)'
 PPPS04D length=3
   label = 'PSI 04D Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum D: Shock/Cardiac (Pop)'
 PPPS04E length=3
   label = 'PSI 04E Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum E: Gastrointestinal (GI) Hemorrhage/Acute Ulcer (Pop)'
 PPPS05 length=3
   label = 'PSI 05 Retained Surgical Item or Unretrieved Device Fragment Count (Pop)'
 PPPS06 length=3
   label = 'PSI 06 Iatrogenic Pneumothorax Rate (Pop)'
 PPPS07 length=3
   label = 'PSI 07 Central Venous Catheter-Related Blood Stream Infection Rate (Pop)'
 PPPS08 length=3
   label = 'PSI 08 In Hospital Fall with Hip Fracture Rate (Pop)'
 PPPS09 length=3
   label = 'PSI 09 Perioperative Hemorrhage or Hematoma Rate (Pop)'
 PPPS10 length=3
   label = 'PSI 10 Postoperative Acute Kidney Injury Requiring Dialysis Rate (Pop)'
 PPPS11 length=3
   label = 'PSI 11 Postoperative Respiratory Failure Rate (Pop)'
 PPPS12 length=3
   label = 'PSI 12 Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate (Pop)'
 PPPS13 length=3
   label = 'PSI 13 Postoperative Sepsis Rate (Pop)'
 PPPS14 length=3
   label = 'PSI 14 Postoperative Wound Dehiscence Rate (Pop)'
 PPPS15 length=3
   label = 'PSI 15 Unrecognized Abdominopelvic Accidental Puncture or Laceration Rate (Pop)'
 PPPS16 length=3
   label = 'PSI 16 Transfusion Reaction Count (Pop)'
 PPPS18 length=3
   label = 'PSI 18 Obstetric Trauma Rate - Vaginal Delivery With Instrument (Pop)'
 PPPS19 length=3
   label = 'PSI 19 Obstetric Trauma Rate - Vaginal Delivery Without Instrument (Pop)'
;
 * -------------------------------------------------------------- ;
 * --- DEFINE AREA LEVEL INDICATORS ----------------------------- ;
 * -------------------------------------------------------------- ;
 
 attrib
 TAPS21 length=8
   label='PSI 21 Retained Surgical Item or Unretrieved Device Fragment Rate (Area Numerator)'
 TAPS22 length=8
   label='PSI 22 Iatrogenic Pneumothorax Rate (Area Numerator)'
 TAPS23 length=8
   label='PSI 23 Central Venous Catheter-Related Blood Stream Infection Rate (Area Numerator)'
 TAPS24 length=8
   label='PSI 24 Postoperative Wound Dehiscence Rate (Area Numerator)'
 TAPS25 length=8
   label='PSI 25 Accidental Puncture or Laceration Rate (Area Numerator)'
 TAPS26 length=8
   label='PSI 26 Transfusion Reaction Rate (Area Numerator)'
 TAPS27 length=8
   label='PSI 27 Perioperative Hemorrhage or Hematoma Rate (Area Numerator)'
;

 * -------------------------------------------------------------- ;
 * --- RE-LABEL DAY DEPENDENT INDICATORS ------------------------ ;
 * -------------------------------------------------------------- ;
 
 %IF &PRDAY. = 0 %THEN %DO;
 label
   TPPS03 = 'PSI 03 Pressure Ulcer Rate-NO PRDAY (Numerator)'
   TPPS08 = 'PSI 08 In Hospital Fall with Hip Fracture Rate-NO PRDAY (Numerator)'
   TPPS09 = 'PSI 09 Perioperative Hemorrhage or Hematoma Rate-NO PRDAY (Numerator)'
   TPPS10 = 'PSI 10 Postoperative Acute Kidney Injury Requiring Dialysis Rate-NO PRDAY (Numerator)'
   TPPS11 = 'PSI 11 Postoperative Respiratory Failure Rate-NO PRDAY (Numerator)'
   TPPS12 = 'PSI 12 Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate-NO PRDAY (Numerator)'
   TPPS14 = 'PSI 14 Postoperative Wound Dehiscence Rate-NO PRDAY (Numerator)'
 ;
 %END;

 %MEND;

 %LBL;

 * -------------------------------------------------------------- ;
 * --- CONSTRUCT AHRQ COMORBIDITY ------------------------------- ;
 * -------------------------------------------------------------- ;

 length
    CHF VALVE PULMCIRC PERIVASC HTN_C
    PARA NEURO CHRNLUNG DM DMCX
    HYPOTHY RENLFAIL LIVER ULCER AIDS
    LYMPH METS TUMOR ARTH OBESE
    WGHTLOSS BLDLOSS ANEMDEF ALCOHOL DRUG
    PSYCH DEPRESS 3;
 
 array ARRY4{27}
    CHF VALVE PULMCIRC PERIVASC HTN_C
    PARA NEURO CHRNLUNG DM DMCX
    HYPOTHY RENLFAIL LIVER ULCER AIDS
    LYMPH METS TUMOR ARTH OBESE
    WGHTLOSS BLDLOSS ANEMDEF ALCOHOL DRUG
    PSYCH DEPRESS;

 *** Declare variables as array elements;

    array DX (&NDX.) $  DX1 - DX&NDX.;
    array DXPOA (&NDX.) $  DXPOA1 - DXPOA&NDX.;
    array PR {&NPR.} PR1-PR&NPR.;
    array COM1 (30)  CHF      VALVE    PULMCIRC PERIVASC
                    HTN      HTNCX    PARA     NEURO    CHRNLUNG
                    DM       DMCX     HYPOTHY  RENLFAIL LIVER
                    ULCER    AIDS     LYMPH    METS     TUMOR
                    ARTH     COAG     OBESE    WGHTLOSS LYTES
                    BLDLOSS  ANEMDEF  ALCOHOL  DRUG     PSYCH
                    DEPRESS ;

    array COM2 (30) $ 8 A1-A30
                    ("CHF"     "VALVE"   "PULMCIRC" "PERIVASC" 
                     "HTN"     "HTNCX"   "PARA"     "NEURO"     "CHRNLUNG" 
                     "DM"      "DMCX"    "HYPOTHY"  "RENLFAIL"  "LIVER" 
                     "ULCER"   "AIDS"    "LYMPH"    "METS"      "TUMOR" 
                     "ARTH"    "COAG"    "OBESE"    "WGHTLOSS"  "LYTES" 
                     "BLDLOSS" "ANEMDEF" "ALCOHOL"  "DRUG"      "PSYCH" 
                     "DEPRESS") ; 

 do J = 1 TO 27;
    ARRY4(J) = 0;
 end ;

 *-- Assign Comorbidity flags for each discharge --* ;
 %include MacLib(PSI_Comorb_Code_v9.sas);
  

 *-- Create IMMUNE flag for risk adjustment --* ;
 if %MDX($IMMUNID.) or %MPR($IMMUNIP.) then IMMUNE=1 ;
 else IMMUNE=0 ;

 * -------------------------------------------------------------- ;
 * --- CONSTRUCT PROVIDER LEVEL INDICATORS ---------------------- ;
 * -------------------------------------------------------------- ;

 * --------------------------------------------------------------------------- ;
 * --- PSI 02: DEATH RATE IN LOW-MORTALITY DIAGNOSIS RELATED GROUPS (DRGS) --- ;
 * --------------------------------------------------------------------------- ;

   if LOWMODR then do;

      TPPS02 = 0;
       
      if DISP in (20) then TPPS02 = 1;

      *** Exclude trauma, immunocompromised state, and cancer;

      if %MDX($TRAUMID.) or %MDX($IMMUNID.) or 
         %MPR($IMMUNIP.) or %MDX($CANCEID.) 
      then TPPS02 = .;

      *** Exclude missing discharge disposition or transfer acute care facility; 

      if DISP le .Z or DISP=2 then TPPS02 = .;

   end ;

 * ----------------------------------- ;
 * --- PSI 03: PRESSURE ULCER RATE --- ;
 * ----------------------------------- ;

 %MACRO PS3;

 if (&PRDAY. = 1  and ORDAY ne . and MPRDAY ne .) then do ;

      if MPRDAY <= ORDAY then TPPS03 = .;

 end ;
 else  do ;

      if %MPR1($DEBRIDP.) then TPPS03 = .;

 end ;

 %MEND;

   if (MEDICDR or SURGIDR) then do ;

      TPPS03 = 0; QPPS03 = 0;
     
   if ICDVER le 25 and %MDX2($DECUBID.) then TPPS03 = 1;
   if ICDVER ge 26 and %MDX2($DECUBID.) and %MDX2($DECUBVD.) then TPPS03 = 1;


   *** Exclude principal diagnosis or numerator event without at least one DECUBID/DECUBVD not POA; 

   if %MDX1($DECUBID.) then TPPS03 = .;
   IF ICDVER LE 25 AND %MDX2Q2($DECUBID.) then QPPS03 = 1;                         /* at least one POA */
   IF ICDVER GE 26 AND %MDX2Q2($DECUBID.) and %MDX2Q2($DECUBVD.) then QPPS03 = 1;  /* at least one POA */
   if ICDVER le 25 and %MDX2Q1($DECUBID.) then QPPS03 = 0;                         /* at least one not POA */
   if ICDVER ge 26 and %MDX2Q1($DECUBID.) and %MDX2Q1($DECUBVD.) then QPPS03 = 0;  /* at least one not POA */

      if SURGIDR and ORCNT > 0 then do ;
 
         *** Exclude if debridement or pedicle graft is the only OR procedure;
         %MPRCNT($DEBRIDP.);
         if ORCNT = MPRCNT then TPPS03 = .;

         *** Exclude if debridement or pedicle graft occurs before or 
             on the same day as the first OR procedure;
         %ORDAY($DEBRIDP.);
         %MPRDAY($DEBRIDP.);
         %PS3;

      end ;

      *** Exclude hemiplegia, paraplagia, quadriplagia;

      if %MDX($HEMIPID_PSI.) then TPPS03 = .;

      *** Exclude spina bifida and anoxic brain damage;

      if %MDX($SPINABD.) then TPPS03 = .;

      *** Exclude MDC 14;

      if MDC in (14) then TPPS03 = .;

      *** Exclude LOS < 3, admitted from acute care facility or LTC;

      if LOS < 3 or 
         ASOURCE in (2,3) or
         POINTOFORIGINUB04 in ('4','5','6') then TPPS03 = .;

      *** Exclude any diagnosis POA of skin disorder;

      if (put(DX1,$poaxmpt.)='0' and put(DX1,$SKINDISD.) = '1' and DXPOA1 in ('Y','W')) or
         (put(DX1,$poaxmpt.)='1' and put(DX1,$SKINDISD.) = '1') or %MDX2Q2($SKINDISD.) then TPPS03 = .;

      *** Set POA flag to missing;

      if TPPS03 = . then QPPS03 = .;
      if TPPS03 = 0 and QPPS03 = 1 then QPPS03 = 0;

   end ;

 * -------------------------------------------------------------------------------------- ;
 * --- PSI 04: DEATH RATE AMONG SURGICAL INPATIENTS WITH SERIOUS TREATABLE CONDITIONS --- ;
 * -------------------------------------------------------------------------------------- ;
 
   if (SURGIDR and ORCNT > 0)   and
      (0 <= PRDAY1 <=2 or ATYPE in (3)) and
      (17 < AGE < 90 or MDC in (14))    then do ;

       /**-- STRATUM A: DEEP VEIN THROMBOSIS/PULMONARY EMBOLISM (DVT/PE) --**/
   if (ICDVER le 26 and (%MDX2($FTR2DX.) and 
       not (%MDX1($FTR2DX.) or %MDX1($OBEMBOL.))))
      or
      (ICDVER ge 27 and (%MDX2($FTR2DXB.) and 
       not (%MDX1($FTR2DXB.) or %MDX1($OBEMBOL.))))
      or
       /**-- STRATUM B: PNEUMONIA --**/
      (%MDX2($FTR3DX.) and 
       not (%MDX1($FTR3DX.) or %MDX1($FTR3EXA.) or %MDX($FTR3EXB.) or %MDX($IMMUNID.) or 
            %MPR($IMMUNIP.) or MDC in (4) or %MPR($LUNGCIP.)))
      or
       /**-- STRATUM C: SEPSIS --**/
      (%MDX2($FTR4DX.) and 
       not (%MDX1($FTR4DX.) or (%MDX1($INFECID.) or %MDX($IMMUNID.) or %MPR($IMMUNIP.)) or LOS < 4))
      or
       /**-- STRATUM D: SHOCK/CARDIAC ARREST --**/
      ((%MDX2($FTR5DX.) or %MPR($FTR5PR.)) and 
       not (%MDX1($FTR5DX.) or %MDX1($FTR5EX.) or %MDX1($HEMORID.) or %MDX1($TRAUMID.) or 
            %MDX1($GASTRID.) or MDC in (4,5) ))
      or
       /**-- STRATUM E: GASTROINTESTINAL (GI) HEMORRHAGE/ACUTE ULCER --**/
      (%MDX2($FTR6DX.) and 
       not (%MDX1($FTR6DX.) or %MDX1($FTR6EX.) or %MDX1($TRAUMID.) or
            %MDX1($ALCHLSM.) or MDC in (6,7)))
   then do ;

      TPPS04 = 0;  

      if DISP in (20) then TPPS04 = 1;

      *** Exclude transfers to an acute care facility;

      if DISP le .Z or DISP=2 then TPPS04 = .;

   end ;

   end ;

 * -------------------------------------------------------------------------------------------------- ;
 * --- PSI 04A: DEATH AMONG SURGICAL INPATIENTS: DEEP VEIN THROMBOSIS/PULMONARY EMBOLISM (DVT/PE) --- ;
 * --- PSI 04B: DEATH AMONG SURGICAL INPATIENTS: PNEUMONIA ------------------------------------------ ;
 * --- PSI 04C: DEATH AMONG SURGICAL INPATIENTS: SEPSIS --------------------------------------------- ;
 * --- PSI 04D: DEATH AMONG SURGICAL INPATIENTS: SHOCK/CARDIAC ARREST ------------------------------- ;
 * --- PSI 04E: DEATH AMONG SURGICAL INPATIENTS: GASTROINTESTINAL (GI) HEMORRHAGE/ACUTE ULCER ------- ;
 * -------------------------------------------------------------------------------------------------- ;

   if (SURGIDR and ORCNT > 0)            and
      (0 <= PRDAY1 <= 2 or ATYPE in (3)) and
      (17 < AGE < 90 or MDC in (14))     then do ;

       /** STRATUM A: DEEP VEIN THROMBOSIS/PULMONARY EMBOLISM (DVT/PE) ***/

        if (ICDVER le 26 and (%MDX2($FTR2DX.) and 
                   not (%MDX1($FTR2DX.) or %MDX1($OBEMBOL.))))
               or
           (ICDVER ge 27 and %MDX2($FTR2DXB.)) then do ;

          TPPS04A = 0; 
          if DISP in (20) then TPPS04A = 1;

          if %MDX1($FTR2DXB.) then TPPS04A = .;
          if %MDX1($OBEMBOL.) then TPPS04A = .;

          *** Exclude Transfers to an acute care facility;

          if DISP le .Z or DISP=2 then TPPS04A = .;

          *** Define ANY and SEVERE risk adjustment variables ;

          XPPS04A_ANY = 0;
          XPPS04A_SEVERE=0;
                do i = 1 to &NDX.;
                    if (ICDVER le 26 and put(DX(i),$FTR2DX.) = '1')
                       or (ICDVER ge 27 and put(DX(i),$FTR2DXB.) = '1') then do;
                        if DXPOA(i) in ('Y','W') then XPPS04A_ANY = 1;
                        if     DX(i) in ("41511" "41513" "41519") then XPPS04A_SEVERE=1;
                    end;    
                end;
    end ;

      /** STRATUM B: PNEUMONIA ***/

      if %MDX2($FTR3DX.) then do ;

          TPPS04B = 0; 
          if DISP in (20) then TPPS04B = 1;

          if %MDX1($FTR3DX.)   then TPPS04B = .;
          if %MDX1($FTR3EXA.)  then TPPS04B = .;
          if %MDX($FTR3EXB.)   then TPPS04B = .;
          if %MDX($IMMUNID.)   then TPPS04B = .;
          if %MPR($IMMUNIP.)   then TPPS04B = .;
          if MDC in (4)        then TPPS04B = .;
          if %MPR($LUNGCIP.)   then TPPS04B = .;

          *** Exclude transfers to an acute care facility;

          if DISP le .Z or DISP=2 then TPPS04B = .;

          *** Define ANY and SEVERE risk adjustment variables ;

          XPPS04B_ANY = 0;
          XPPS04B_SEVERE=0;
                do i = 1 to &NDX.;
                    if put(DX(i),$FTR3DX.) = '1' then do;
                        if DXPOA(i) in ('Y','W') then XPPS04B_ANY = 1;
                        if     DX(i) in ("4820" "4821" "48240" "48241" "48242" "48249" "48281" "48282" "48283" "5070") then XPPS04B_SEVERE=1;
                    end;    
                end;
      end ;


      /** STRATUM C: SEPSIS ***/

       if %MDX2($FTR4DX.) then do ;

          TPPS04C = 0; 
          if DISP in (20) then TPPS04C = 1;

          if %MDX1($FTR4DX.) then TPPS04C = .;
          if %MDX1($INFECID.) then TPPS04C = .;
          if %MDX($IMMUNID.)  then TPPS04C = .;
          if %MPR($IMMUNIP.)  then TPPS04C = .;
          if LOS lt 4 then TPPS04C=.;

          *** Exclude transfers to an acute care facility;

          if DISP le .Z or DISP=2 then TPPS04C = .;

          *** Define ANY and SEVERE risk adjustment variables ;

          XPPS04C_ANY = 0;
          XPPS04C_SEVERE=0;
                do i = 1 to &NDX.;
                    if put(DX(i),$FTR4DX.) = '1' then do;
                        if DXPOA(i) in ('Y','W') then XPPS04C_ANY = 1;
                        if     DX(i) in ("78552" "99592" "99802") then XPPS04C_SEVERE=1;
                    end;    
                end;
       end ;


      /** STRATUM D: SHOCK/CARDIAC ARREST ***/

       if %MDX2($FTR5DX.) or %MPR($FTR5PR.) then do ;

          TPPS04D = 0;
          if DISP in (20) then TPPS04D = 1;

          if %MDX1($FTR5DX.)  then TPPS04D = .;
          if %MDX1($TRAUMID.) then TPPS04D = .;
          if %MDX1($HEMORID.) then TPPS04D = .;
          if %MDX1($GASTRID.) then TPPS04D = .;
          if %MDX1($FTR5EX.)  then TPPS04D = .;
          if MDC in (4,5)     then TPPS04D = .;
          
             *** Exclude Transfers to an acute care facility;

          if DISP le .Z or DISP=2 then TPPS04D = .;

          *** Define ANY and SEVERE risk adjustment variables ;

          XPPS04D_ANY = 0;
          XPPS04D_SEVERE=0;
                do i = 1 to &NDX.;
                    if put(DX(i),$FTR5DX.) = '1' then do;
                        if DXPOA(i) in ('Y','W') then XPPS04D_ANY = 1;
                        if     DX(i) in ("4275" "7991") then XPPS04D_SEVERE=1;
                    end;    
                end;
                do i = 1 to &NPR. until (XPPS04D_SEVERE=1);
                    if     PR(i) in ("9393" "9960" "9963") then XPPS04D_SEVERE=1;    
                end;                
       end ;


       /** STRATUM E: GASTROINTESTINAL (GI) HEMORRHAGE/ACUTE ULCER ***/

       if %MDX2($FTR6DX.) then do ;

          TPPS04E = 0; 
          if DISP in (20) then TPPS04E = 1;

          if %MDX1($FTR6DX.)  then TPPS04E = .;
          if %MDX1($FTR6EX.)  then TPPS04E = .;
          if %MDX1($TRAUMID.) then TPPS04E = .;
          if %MDX1($ALCHLSM.) then TPPS04E = .;
          if MDC in (6,7)     then TPPS04E = .;

          *** Exclude transfers to an acute care facility;

          if DISP le .Z or DISP=2 then TPPS04E = .;
        
          *** Define ANY and SEVERE risk adjustment variables ;

          XPPS04E_ANY = 0;
          XPPS04E_SEVERE=0;
                do i = 1 to &NDX.;
                    if put(DX(i),$FTR6DX.) = '1' then do;
                        if DXPOA(i) in ('Y','W') then XPPS04E_ANY = 1;
                        if     DX(i) in (    "4560" "45620" "53110" "53111" "53120" "53121" 
                                        "53210" "53211" "53220" "53221" "53310" "53311" 
                                        "53320" "53321" "53410" "53411" "53420" "53421") 
                        then XPPS04E_SEVERE=1;    
                    end;    
                end;
       end ;


 *-------------------------------------------------------------------*;
 *---------- MAKE STRATA MUTUALLY EXCLUSIVE AND EXHAUSTIVE        ---*;  
 *-------------------------------------------------------------------*;
       if TPPS04D ne . then do ;
                       TPPS04C = .;XPPS04C_ANY = .;XPPS04C_SEVERE = .;
                       TPPS04B = .;XPPS04B_ANY = .;XPPS04B_SEVERE = .;
                       TPPS04A = .;XPPS04A_ANY = .;XPPS04A_SEVERE = .;
                       TPPS04E = .;XPPS04E_ANY = .;XPPS04E_SEVERE = .;
       end ;

       else  if TPPS04C ne . then do ;
                       TPPS04D = .;XPPS04D_ANY = .;XPPS04D_SEVERE = .;
                       TPPS04B = .;XPPS04B_ANY = .;XPPS04B_SEVERE = .;
                       TPPS04A = .;XPPS04A_ANY = .;XPPS04A_SEVERE = .;
                       TPPS04E = .;XPPS04E_ANY = .;XPPS04E_SEVERE = .;
       end ;
       else  if TPPS04B ne . then do ;
                       TPPS04D = .;XPPS04D_ANY = .;XPPS04D_SEVERE = .;
                       TPPS04C = .;XPPS04C_ANY = .;XPPS04C_SEVERE = .;
                       TPPS04A = .;XPPS04A_ANY = .;XPPS04A_SEVERE = .;
                       TPPS04E = .;XPPS04E_ANY = .;XPPS04E_SEVERE = .;
       end ;

       else  if TPPS04A ne . then do ;
                       TPPS04D = .;XPPS04D_ANY = .;XPPS04D_SEVERE = .;
                       TPPS04C = .;XPPS04C_ANY = .;XPPS04C_SEVERE = .;
                       TPPS04B = .;XPPS04B_ANY = .;XPPS04B_SEVERE = .;
                       TPPS04E = .;XPPS04E_ANY = .;XPPS04E_SEVERE = .;
       end ;

       else  if TPPS04E ne . then do ;
                       TPPS04D = .;XPPS04D_ANY = .;XPPS04D_SEVERE = .;
                       TPPS04C = .;XPPS04C_ANY = .;XPPS04C_SEVERE = .;
                       TPPS04B = .;XPPS04B_ANY = .;XPPS04B_SEVERE = .;
                       TPPS04A = .;XPPS04A_ANY = .;XPPS04A_SEVERE = .;
       end ;

 *** Write a message to the log if the strata are not exhaustive;

       if (TPPS04 EQ .)  and 
          ((TPPS04A ne .) or 
           (TPPS04B ne .) or 
           (TPPS04C ne .) or 
           (TPPS04D ne .) or 
           (TPPS04E ne .)) then do ;
          put "ERROR IN STRATUM ASSIGNMENT FOR PSI 4";
          put "Record is not in denominator of the parent measure but is in one or more strata";
          put "KEY: " KEY 
              "TPPS04:"  TPPS04 
              "TPPS04A:" TPPS04A 
              "TPPS04B:" TPPS04B 
              "TPPS04C:" TPPS04C 
              "TPPS04D:" TPPS04D 
              "TPPS04E:" TPPS04E;
       end ;

 *** Ensure that every record in the parent measure is in at least one stratum;

       if  (TPPS04 ne .)   and 
           ((TPPS04A EQ .) and 
            (TPPS04B EQ .) and 
            (TPPS04C EQ .) and 
            (TPPS04D EQ .) and 
            (TPPS04E EQ .)) then do ;
          put "ERROR IN STRATUM ASSIGNMENT FOR PSI 4";
          put "OBS is in denominator of the parent measure but not in any stratum";
          put "KEY: " KEY 
              "TPPS04:"  TPPS04 
              "TPPS04A:" TPPS04A 
              "TPPS04B:" TPPS04B 
              "TPPS04C:" TPPS04C 
              "TPPS04D:" TPPS04D
              "TPPS04E:" TPPS04E;
       end ;
     end;

 * --------------------------------------------------------------------------- ;
 * --- PSI 05: RETAINED SURGICAL ITEM OR UNRETRIEVED DEVICE FRAGMENT COUNT --- ;
 * --------------------------------------------------------------------------- ;

   if (MEDICDR or SURGIDR) then do ;

      TPPS05 = .; QPPS05 = .;

      if %MDX2($FOREIID.) then TPPS05 = 1;
      if %MDX2Q1($FOREIID.) then QPPS05 = 0;

      *** Exclude principal diagnosis; 

      if %MDX1($FOREIID.) then TPPS05 = .;
      if %MDX2Q2($FOREIID.) then QPPS05 = 1;

      *** Set POA flag to missing;

      if TPPS05 = . then QPPS05 = .;
      if TPPS05 = 0 and QPPS05 = 1 then QPPS05 = 0;

   end ;

   * -------------------------------------------- ;
   * --- PSI 06: IATROGENIC PNEUMOTHORAX RATE --- ;
   * -------------------------------------------- ;

   if (MEDICDR or SURGIDR) then do ;

      TPPS06 = 0; QPPS06 = 0;

      if %MDX2($IATROID.) then TPPS06 = 1;
      if %MDX2Q1($IATROID.) then QPPS06 = 0;

      *** Exclude principal diagnosis; 

      if %MDX1($IATROID.) then TPPS06 = .;
      if %MDX2Q2($IATROID.) then QPPS06 = 1;

      *** Exclude chest trauma, pleural effusion or MDC 14;

      if %MDX($CTRAUMD.) or %MDX($PLEURAD.) or MDC in (14)
      then TPPS06 = .;

      *** Exclude thoracic surgery, lung or pleural biopsy,
          cardiac surgery or diaphragmatic surgery repair;

      if %MPR($THORAIP.) or %MPR($LUNGBIP.) or %MPR($CARDSIP.) or 
         %MPR($DIAPHRP.)
      then TPPS06 = .;

      *** Set POA flag to missing;

      if TPPS06 = . then QPPS06 = .;
      if TPPS06 = 0 and QPPS06 = 1 then QPPS06 = 0;

  end ;

   * --------------------------------------------------------------------------- ;
   * --- PSI 07: CENTRAL VENOUS CATHETER-RELATED BLOOD STREAM INFECTION RATE --- ;
   * --------------------------------------------------------------------------- ;

   if (MEDICDR or SURGIDR) then do ;

      TPPS07 = 0; QPPS07 = 0; 

       if ((ICDVER ge 25 and ICDVER le 28) and (%MDX2($IDTMC2D.))) or
          (ICDVER ge 29 and (%MDX2($IDTMC3D_PSI.)))
      then TPPS07 = 1;

       if ((ICDVER ge 25 and ICDVER le 28) and (%MDX2Q1($IDTMC2D.))) or
          (ICDVER ge 29 and (%MDX2Q1($IDTMC3D_PSI.)))
      then QPPS07 = 0;

        *** Exclude principal diagnosis; 

       if ((ICDVER ge 25 and ICDVER le 28) and (%MDX1($IDTMC2D.))) or
          (ICDVER ge 29 and (%MDX1($IDTMC3D_PSI.)))
      then TPPS07= .;

       if ((ICDVER ge 25 and ICDVER le 28) and (%MDX2Q2($IDTMC2D.))) or
          (ICDVER ge 29 and (%MDX2Q2($IDTMC3D_PSI.)))
      then QPPS07= 1;

      *** Exclude immunocompromised state and Cancer;

      if %MDX($IMMUNID.) or %MPR($IMMUNIP.) or %MDX($CANCEID.)  
      then TPPS07 = .;
 
      *** Exclude LOS < 2; 
      if LOS < 2 then TPPS07 = .;

      *** Set POA flag to missing;

      if TPPS07 = . then QPPS07 = .;
      if TPPS07 = 0 and QPPS07 = 1 then QPPS07 = 0;

   end ;

 * ------------------------------------------------------- ;
 * --- PSI 08: IN HOSPITAL FALL WITH HIP FRACTURE RATE --- ;
 * ------------------------------------------------------- ;

   if (SURGIDR and ORCNT > 0) or MEDICDR then do ;

      TPPS08 = 0; QPPS08 = 0; 
  
      if %MDX2($HIPFXID.) then TPPS08 = 1;
      if %MDX2Q1($HIPFXID.) then QPPS08 = 0;

      *** Exclude principal diagnosis;

      if %MDX1($HIPFXID.) then TPPS08 = .;
      if %MDX2Q2($HIPFXID.) then QPPS08 = 1;

      *** Excludes seizure, syncope, stroke, coma, cardiac arrest,
          poisoning, trauma, delirium and other psychoses, anoxic
          brain injury;

      if %MDX1($SEIZUID.) or %MDX1($SYNCOID.) or %MDX1($STROKID.) or
         %MDX1($COMAID.)  or %MDX1($CARDIID.) or %MDX1($POISOID.) or
         %MDX1($TRAUMID.) or %MDX1($DELIRID.) or %MDX1($ANOXIID.)
      then TPPS08 = .;

      *** Excludes metastatic cancer, lymphoid malignancy, 
          bone malignancy, self-inflicted injury;
 
      if %MDX($METACID.) or %MDX($LYMPHID.) or %MDX($BONEMID.) OR
         %MDX($SELFIID.)
      then TPPS08 = .;

      *** Exclude MDC 8 and 14;

      if MDC in (8, 14) then TPPS08 = .;

      *** Set POA flag to missing;

      if TPPS08 = . then QPPS08 = .;

      if TPPS08 = 0 and QPPS08 = 1 then QPPS08 = 0;

   end ;

  * ---------------------------------------------------------- ;
  * --- PSI 09: PERIOPERATIVE HEMORRHAGE OR HEMATOMA RATE  --- ;
  * ---------------------------------------------------------- ;

 %MACRO PS9;

 if (&PRDAY. = 1 and ORDAY ne . and MPRDAY ne .) then do ;

      if %MDX2($POHMRI2D.) and MPRDAY < ORDAY
      then TPPS09 = .;

 end ;
 else  do ;

      if %MDX2($POHMRI2D.) and %MPR1($HEMOTH2P.)
      then TPPS09 = .;

 end ;

 %MEND;

   if SURGIDR and ORCNT > 0 then do ;

      TPPS09 = 0; QPPS09 = 0; 

      if %MDX2($POHMRI2D.) and 
          (%MPR($HEMOTH2P.)) 
      then TPPS09 = 1;

      if %MDX2Q1($POHMRI2D.) and 
         (%MPR($HEMOTH2P.)) 
      then QPPS09 = 0;

      *** Exclude principal diagnosis; 

      if %MDX1($POHMRI2D.) then TPPS09 = .;

      if %MDX2Q2($POHMRI2D.) then QPPS09 = 1;

      *** Exclude if control of post-operative hemorrhage or miscellaneous hemorrhage 
          hematoma-related procedure are the only OR procedures; 
      %MPRCNT($HEMOTH2P.);

      if ORCNT = MPRCNT then TPPS09 = .;

      *** Exclude if control of post-operative hemorrhage or miscellaneous hemorrhage or
          hematoma-related procedure occurs before the first OR procedure;
      %ORDAY($HEMOTH2P.);
        %MPRDAY($HEMOTH2P.);
      %PS9;

      *** Exclude MDC 14;

      if MDC in (14) then TPPS09 = .;

       *** Exclude coagulation disorders;

      if %MDX($COAGDID.) then TPPS09 = . ;

      *** Set POA flag to missing;

      if TPPS09 = . then QPPS09 = .;

      if TPPS09 = 0 and QPPS09 = 1 then QPPS09 = 0;

   end ;


  * -------------------------------------------------------------------------- ;
  * --- PSI 10: POSTOPERATIVE ACUTE KIDNEY INJURY REQUIRING DIALYSIS RATE  --- ;
  * -------------------------------------------------------------------------- ;

 %MACRO PS10;

 if (&PRDAY. = 1  and ORDAY ne . and MPRDAY ne .) then do ;

      if %MDX2($PHYSIDB.) and MPRDAY <= ORDAY then TPPS10 = .;

 end ;
 else  do ;

      if %MDX2($PHYSIDB.) and %MPR1($DIALYIDP.) then TPPS10 = .;

 end ;
 %MEND;

 
   if SURGIDR and ORCNT > 0 and ATYPE in (3) then do ;

      TPPS10 = 0; QPPS10 = 0;

      IF (%MDX2($PHYSIDB.) AND %MPR($DIALYIP.))
      then TPPS10 = 1;

      IF (%MDX2Q1($PHYSIDB.) AND %MPR($DIALYIP.))
      then QPPS10 = 0;   

      *** Exclude principal diagnosis; 

      if %MDX1($PHYSIDB.) then TPPS10 = .;
      if %MDX2Q2($PHYSIDB.) then QPPS10 = 1;  

      *** Exclude if dialysis access or procedure occurs before or on the same
          day as the first OR procedure;
      %ORDAY($BLANK.);
      %MPRDAY($DIALYIDP.);
      %PS10;

     
      if %MDX2($PHYSIDB.) and (%MDX1($CARDRID.) or %MDX1($CARDIID.) or 
         %MDX1($SHOCKID.) or %MDX1($CRENLFD.))
      then TPPS10 = .;

      if %MDX2($PHYSIDB.) and (%MDX2Q2($CARDRID.) or %MDX2Q2($CARDIID.) or  
         %MDX2Q2($SHOCKID.) or %MDX2Q2($CRENLFD.))
      then QPPS10 = 1;

      *** Exclude MDC 14;

      if MDC in (14) then TPPS10 = .;

      *** Exclude hospitalizations with principal diagnosis of urinary tract obstruction; 

      if %MDX1($URINARYOBSID.) then TPPS10 = .;
  
      *** Set POA flag to missing;

      if TPPS10 = . then QPPS10 = .;
      if TPPS10 = 0 and QPPS10 = 1 then QPPS10 = 0;

   end ;


  * ------------------------------------------------------ ;
  * --- PSI 11: PERIOPERATIVE RESPIRATORY FAILURE RATE --- ;
  * ------------------------------------------------------ ;

 %MACRO PS11N(FMT,DAYS);

    %MPRDAY(&FMT.);

    if (&PRDAY. = 1  and ORDAY ne . and MPRDAY ne .) then do ;

       if MPRDAY >= ORDAY + &DAYS. then TPPS11 = 1;

    end ;
    else  do ;

       if %MPR2(&FMT.) then TPPS11 = 1;

    end ;

 %MEND;

 %MACRO PS11;

 if (&PRDAY. = 1  and ORDAY ne . and MPRDAY ne .) then do ;

      if MPRDAY < ORDAY then TPPS11 = .;

 end ;
 else  do ;

      if %MPR1($TRACHIP.) then TPPS11 = .;

 end ;

 %MEND;

   if SURGIDR and ORCNT > 0 and ATYPE in (3) then do ;

      TPPS11 = 0; QPPS11 = 0;

      if (ICDVER le 28 and %MDX2($ACURFID.))   or (ICDVER ge 29 and %MDX2($ACURF2D.))   then TPPS11 = 1;
      if (ICDVER le 28 and %MDX2Q1($ACURFID.)) or (ICDVER ge 29 and %MDX2Q1($ACURF2D.)) then QPPS11 = 0;

      *** Include in numerator if reintubation procedure occurs on the same day or
          one or two days after the first OR procedure;
 
      %ORDAY($BLANK.);
      %PS11N($PR9604P.,1);
      %PS11N($PR9670P.,2);
      %PS11N($PR9671P.,2);
      %PS11N($PR9672P.,0);

      *** Exclude principal diagnosis; 

      if (ICDVER le 28 and %MDX1($ACURFID.))   or (ICDVER ge 29 and %MDX1($ACURF3D.))    then TPPS11 = .;
      if (ICDVER le 28 and %MDX2Q2($ACURFID.)) or (ICDVER ge 29 and %MDX2Q2($ACURF3D.))  then QPPS11 = 1;

      *** Exclude if tracheostomy procedure is the only OR procedure;

      %MPRCNT($TRACHIP.);
      if ORCNT = MPRCNT then TPPS11 = .;

      *** Exclude if tracheostomy procedure occurs before the 
          first or procedure;
      %ORDAY($TRACHIP.);
      %MPRDAY($TRACHIP.);
      %PS11;

      *** Exclude neuromuscular disorders;
 
      if %MDX($NEUROMD.) then TPPS11 = .;

      *** Exclude MDC 4, 5, 14;
 
      if MDC in (4,5,14) then TPPS11 = .;

      *** Exclude craniofacial anomalies;
      if %MPR($NUCRANP.) or
         (%MPR($CRANI2P_PSI.) and %MDX($CRANIID.)) 
      then TPPS11 = .;

      *** Exclude esophageal resection procedure;
      if %MPR($PRESOPP.) or %MPR($PRESO2P.) 
      then TPPS11 = .;

      *** Exclude lung cancer procedure;
      if %MPR($LUNGCIP.) then TPPS11 = .;
   
      *** Exclude diagnosis of degenerative neurological disorder;
      if %MDX($DGNEUID.) then TPPS11 = .;

      *** Exclude lung transplant procedure;
      if %MPR($LUNGTRANSP.) then TPPS11 = .;

      *** Set POA flag to missing;

      if TPPS11 = . then QPPS11 = .;
      if TPPS11 = 0 and QPPS11 = 1 then QPPS11 = 0;

   end ;

   
  * ----------------------------------------------------------------------------- ;
  * --- PSI 12: PERIOPERATIVE PULMONARY EMBOLISM OR DEEP VEIN THROMBOSIS RATE --- ;
  * ----------------------------------------------------------------------------- ;
 
 %MACRO PS12;
 
   if (&PRDAY. = 1  and ORDAY ne . and MPRDAY ne .) then do ;
 
        if MPRDAY <= ORDAY then TPPS12 = .;
 
   end ;
   else  do ;
 
        if %MPR1($VENACIP.) then TPPS12 = .;
 
   end ; 
   if %MPR($ECMOP.) then TPPS12 = . ;

 %MEND;
 
   if SURGIDR and ORCNT > 0 then do ;
 
      TPPS12 = 0; QPPS12 = 0;
 
      if (ICDVER le 26 and %MDX2($DEEPVID.)) or %MDX2($PULMOID.) then TPPS12 = 1;
      if (ICDVER ge 27 and %MDX2($DEEPVIB.)) or %MDX2($PULMOID.) then TPPS12 = 1;
      if %MDX2Q1($DEEPVID.) or %MDX2Q1($PULMOID.) then QPPS12 = 0;
 
      *** Exclude principal diagnosis;
 
      if (ICDVER le 26 and %MDX1($DEEPVID.)) or %MDX1($PULMOID.) then TPPS12 = .;
      if (ICDVER ge 27 and %MDX1($DEEPVIB.)) or %MDX1($PULMOID.) then TPPS12 = .;
      if (ICDVER le 26 and %MDX2Q2($DEEPVID.)) or %MDX2Q2($PULMOID.) then QPPS12 = 1;
      if (ICDVER ge 27 and %MDX2Q2($DEEPVIB.)) or %MDX2Q2($PULMOID.) then QPPS12 = 1;
 
 
      *** Exclude if interruption of vena cava occurs before or on the
          same day as the first OR procedure;
 
      %ORDAY($VENACIP.);
      %MPRDAY($VENACIP.);
      %PS12;

      POSTLOS = LOS - ORDAY;
 
      *** Exclude MDC 14;
 
      if MDC in (14) then TPPS12 = .;

      *** Exclude any diagnosis POA of acute brain and or spinal injury;

       if (put(DX1,$poaxmpt.)='0' and put(DX1,$NEURTRAD.) = '1' and DXPOA1 in ('Y','W')) or
         (put(DX1,$poaxmpt.)='1' and put(DX1,$NEURTRAD.) = '1') or %MDX2Q2($NEURTRAD.) then TPPS12 = .;
 

      *** Set POA flag to missing;
 
      if TPPS12 = . then QPPS12 = .;
      if TPPS12 = 0 and QPPS12 = 1 then QPPS12 = 0;
 
   end ;
       

   * ----------------------------------------- ;
   * --- PSI 13: POSTOPERATIVE SEPSIS RATE --- ;
   * ----------------------------------------- ;

   if SURGIDR and ORCNT > 0 AND ATYPE IN (3) then do;

      TPPS13 = 0; QPPS13 = 0;

      if (ICDVER ge 25 and %MDX2($SEPTI2D_PSI.))
      then TPPS13 = 1;

      if (ICDVER ge 25 and %MDX2Q1($SEPTI2D_PSI.))
      then QPPS13 = 0;

      *** Exclude principal diagnosis; 

      if (ICDVER ge 25 and %MDX1($SEPTI2D_PSI.))
      then TPPS13 = .;

      if (ICDVER ge 25 and %MDX2Q2($SEPTI2D_PSI.))
      then QPPS13 = 1;

      *** Exclude infection;

      if %MDX1($INFECID.)   then TPPS13 = .;

      if %MDX2Q2($INFECID.)  then QPPS13 = 1;
       
      *** Exclude MDC 14;

      if MDC in (14) then TPPS13 = .;

      *** Set POA flag to missing;

      if TPPS13 = . then QPPS13 = .;
      if TPPS13 = 0 and QPPS13 = 1 then QPPS13 = 0;

   end ;

 * --------------------------------------------------- ;
 * --- PSI 14: POSTOPERATIVE WOUND DEHISCENCE RATE --- ;
 * --------------------------------------------------- ;

 %MACRO PS14;

 if (&PRDAY. = 1 and MPRDAYA ne . and MPRDAYB ne .) then do ;

      if MPRDAYB <= MPRDAYA then TPPS14 = .;

 end ;
 else  do ;

     if %MPR1($RECLOIP.) then TPPS14 = .;

 end ;

 %MEND;

   if (ICDVER ge 25 and %MPR($ABDOM14IP.)) 
      
   then do ;

      TPPS14 = 0; QPPS14 = 0;

      if %MPR($RECLOIP.) then TPPS14 = 1;

      *** Exclude if wound reclosure occurs before or on the same
          day as the first abdominopelvic procedure;

      %MPRDAY($ABDOM14IP.); MPRDAYA = MPRDAY;
      %MPRDAY($RECLOIP.); MPRDAYB = MPRDAY;
      %PS14;

      *** Exclude primary Dx or secondary Dx POA of disruption of internal operation wound ;
      if %MDX1($ABWALLCD.) or %MDX2Q2($ABWALLCD.) then TPPS14 = .;

      *** Exclude immunocompromised state;

      if %MDX($IMMUNID.) or %MPR($IMMUNIP.) 
      then TPPS14 = .;
 
      *** Exclude LOS < 2; 
      if LOS < 2 then TPPS14 = .;

      *** Exclude MDC 14;

      if MDC in (14) then TPPS14 = .;

      *** Set POA flag to missing;

      if TPPS14 = . then QPPS14 = .;
      if TPPS14 = 0 and QPPS14 = 1 then QPPS14 = 0;

   end ;

  * ---------------------------------------------------------------------------------- ;
  * --- PSI 15: UNRECOGNIZED ABDOMINOPELVIC ACCIDENTAL PUNCTURE OR LACERATION RATE --- ;
  * ---------------------------------------------------------------------------------- ;

  %MACRO PS15(FMT1,FMT2);
    PR1DAY = .;
    PR2DAY = .;
    PR1INDX = 0;
    MPRDAYCD = 0;
    %DO I = 1 %TO &NPR.;
       if put(PR&I.,&FMT1.) = '1' and (PRDAY&I. GT .Z) then do;
          if PR1DAY LE .Z then do;
             PR1DAY = PRDAY&I;
             PR1INDX = &I;
          end;
          else if PR1DAY > PRDAY&I. then do;
             PR1DAY = PRDAY&I;
             PR1INDX = &I;
         end;
       end;
    %END;

    %DO I = 1 %TO &NPR.;
       if put(PR&I.,&FMT2.) = '1' and (PRDAY&I. GT .Z) and (&I NE PR1INDX) then do;
          if PR2DAY LE .Z then do;
             PR2DAY = PRDAY&I;
          end;
          else if PR2DAY > PRDAY&I. then do;
             PR2DAY = PRDAY&I;
          end;
       end;
    %END;

    if (PR2DAY GT .Z) and (PR1DAY GT .Z) and (PR2DAY > PR1DAY) then MPRDAYCD = 1;

  %MEND;

   if (MEDICDR or SURGIDR) and %MPR($ABDOM15IP.) then do;

      TPPS15 = 0; QPPS15 = 0;

      if %MDX2($TECHNID.) then do;
        %PS15($ABDOM15IP.,$ABDOM15IP.);
        if MPRDAYCD = 1 then TPPS15 = 1;
        if %MDX2Q1($TECHNID.) then QPPS15 = 0;
      end;

      *** Exclude principal diagnosis;
      if %MDX1($TECHNID.) then TPPS15 = .;
      if %MDX2Q2($TECHNID.) then QPPS15 = 1;

      *** Exclude MDC 14;
      if MDC IN (14) then TPPS15 = .;

      *** Set POA flag to missing;
      if TPPS15 = . then QPPS15 = .;
      if TPPS15 = 0 and QPPS15 = 1 then QPPS15 = 0;

   end;

  * ------------------------------------------ ;
  * --- PSI 16: TRANSFUSION REACTION COUNT --- ;
  * ------------------------------------------ ;

   if (MEDICDR or SURGIDR) then do ;

      TPPS16 = .; QPPS16 = .;
 
      if %MDX2($TRANFID.) then TPPS16 = 1;
      if %MDX2Q1($TRANFID.) then QPPS16 = 0;

      *** Exclude principal diagnosis; 

      if %MDX1($TRANFID.) then TPPS16 = .;
      if %MDX2Q2($TRANFID.) then QPPS16 = 1;

       *** Set POA flag to missing;

      if TPPS16 = . then QPPS16 = .;
      if TPPS16 = 0 and QPPS16 = 1 then QPPS16 = 0;

   end ;

  * ----------------------------------------------------------------------------- ;
  * --- PSI 18: OBSTETRIC TRAUMA RATE - VAGINAL DELIVERY WITH INSTRUMENTATION --- ;
  * ----------------------------------------------------------------------------- ;

   if %MDR($PRVAG2G.) and %MPR($INSTRIP.)
   then do ;

      TPPS18 = 0;

      if %MDX($OBTRAID.) then TPPS18 = 1;

   end ;

   * -------------------------------------------------------------------------------- ;
   * --- PSI 19: OBSTETRIC TRAUMA RATE - VAGINAL DELIVERY WITHOUT INSTRUMENTATION --- ;
   * -------------------------------------------------------------------------------- ;

   if  %MDR($PRVAG2G.) and not %MPR($INSTRIP.) 
   then do ;

      TPPS19 = 0;

      if %MDX($OBTRAID.) then TPPS19 = 1;

   end ;

 * -------------------------------------------------------------- ;
 * --- CONSTRUCT AREA LEVEL INDICATORS -------------------------- ;
 * -------------------------------------------------------------- ;
  
  * -------------------------------------------------------------------------- ;
  * --- PSI 21: RETAINED SURGICAL ITEM OR UNRETRIEVED DEVICE FRAGMENT RATE --- ;
  * -------------------------------------------------------------------------- ;

   if (MEDICDR or SURGIDR) then do ;

      TAPS21 = .;

      if %MDX($FOREIID.) then TAPS21 = 1;

   end ;

  * -------------------------------------------- ;
  * --- PSI 22: IATROGENIC PNEUMOTHORAX RATE --- ;
  * -------------------------------------------- ;

   if (MEDICDR or SURGIDR) then do ;

      TAPS22 = 0;

      if %MDX($IATROID.) then TAPS22 = 1;
   
      *** Exclude chest trauma, pleural effusion or MDC 14;

      if %MDX($CTRAUMD.) or %MDX($PLEURAD.) or MDC in (14)
      then TAPS22 = .;

      *** Exclude thoracic surgery, lung or pleural biopsy,
          cardiac surgery or diaphragmatic surgery repair;

      if %MPR($THORAIP.) or %MPR($LUNGBIP.) or %MPR($CARDSIP.) or 
         %MPR($DIAPHRP.)
      then TAPS22 = .;

   end ;

   * --------------------------------------------------------------------------- ;
   * --- PSI 23: CENTRAL VENOUS CATHETER-RELATED BLOOD STREAM INFECTION RATE --- ;
   * --------------------------------------------------------------------------- ;

   if (MEDICDR or SURGIDR) then do ;

      TAPS23 = 0;

      if ((ICDVER ge 25 and ICDVER le 28) and (%MDX($IDTMC2D.))) or
          (ICDVER ge 29 and (%MDX($IDTMC3D_PSI.)))

      then TAPS23 = 1;

  
      *** Exclude immunocompromised state and cancer;
 
      if %MDX($IMMUNID.) or %MPR($IMMUNIP.) or %MDX($CANCEID.)  
      then TAPS23 = .;

   end ;

   * --------------------------------------------------- ;
   * --- PSI 24: POSTOPERATIVE WOUND DEHISCENCE RATE --- ;
   * --------------------------------------------------- ;

   TAPS24 = 0;

   if %MPR($RECLOIP.) then TAPS24 = 1;

   *** Exclude immunocompromised state;
   if %MDX($IMMUNID.) or %MPR($IMMUNIP.) 
   then TAPS24 = .;
 
   *** Exclude MDC 14;

   if MDC in (14) then TAPS24 = .;

   * ------------------------------------------------------ ;
   * --- PSI 25: ACCIDENTAL PUNCTURE OR LACERATION RATE --- ;
   * ------------------------------------------------------ ;

   if (MEDICDR or SURGIDR) and %MPR($ABDOM15IP.) then do;

      TAPS25 = 0;

      if %MDX2($TECHNID.) then do;
        %PS15($ABDOM15IP.,$ABDOM15IP.);
        if MPRDAYCD = 1 then TAPS25 = 1;
      end;

      *** Exclude MDC 14;
      if MDC in (14) then TAPS25 = .;

   end ;


   * ------------------------------------------ ;
   * ---  PSI 26: TRANSFUSION REACTION RATE --- ;
   * ------------------------------------------ ;

   if (MEDICDR or SURGIDR) then do ;

      TAPS26 = .;
 
      if %MDX($TRANFID.) then TAPS26 = 1;

   end ;


   * ---------------------------------------------------------- ;
   * ---  PSI 27: POSTOPERATIVE HEMORRHAGE OR HEMATOMA RATE --- ;
   * ---------------------------------------------------------- ;

   TAPS27 = 0;

   if (%MDX($POHMRI2D.) and %MPR($HEMOTH2P.)) 
   then TAPS27 = 1;

   *** Exclude MDC 14;

   if MDC in (14) then TAPS27 = .;

   *** Exclude coagulation disorders;

      if %MDX($COAGDID.) then TAPS27 = . ;


 * -------------------------------------------------------------- ;
 * --- IDENTIFY TRANSFERS --------------------------------------- ;
 * -------------------------------------------------------------- ;

 * --- TRANSFER FROM ANOTHER ACUTE ---------------- ;
 if ASOURCE in (2) or POINTOFORIGINUB04 in ('4') then TRNSFER = 1;
 else  TRNSFER = 0;

 * --- Sync QPPS, TPPS and PPPS VALUES --- ;

    if QPPS03 in (.,1) then TPPS03 = .;
    if QPPS05 in (.,1) then TPPS05 = .;
    if QPPS06 in (.,1) then TPPS06 = .;
    if QPPS07 in (.,1) then TPPS07 = .;
    if QPPS08 in (.,1) then TPPS08 = .;
    if QPPS09 in (.,1) then TPPS09 = .;
    if QPPS10 in (.,1) then TPPS10 = .;
    if QPPS11 in (.,1) then TPPS11 = .;
    if QPPS12 in (.,1) then TPPS12 = .;
    if QPPS13 in (.,1) then TPPS13 = .;
    if QPPS14 in (.,1) then TPPS14 = .;
    if QPPS15 in (.,1) then TPPS15 = .;
    if QPPS16 in (.,1) then TPPS16 = .;

 array TPPS TPPS02-TPPS04 TPPS04A TPPS04B TPPS04C TPPS04D TPPS04E TPPS05-TPPS16 TPPS18 TPPS19;
 array PPPS 3 PPPS02-PPPS04 PPPS04A PPPS04B PPPS04C PPPS04D PPPS04E PPPS05-PPPS16 PPPS18 PPPS19;

 do OVER TPPS;
    if TPPS ge 0 then PPPS = 1;
    else PPPS = .;
 end ;

 * -------------------------------------------------------------- ;
 * --- LABELS --------------------------------------------------- ;
 * -------------------------------------------------------------- ;

 label
    TRNSFER  = 'TRANSFER FROM ACUTE'
 ;

run;

proc Contents data=OUTMSR.&OUTFILE_MEAS. position;
run;

proc Print data=OUTMSR.&OUTFILE_MEAS. (obs=24);
title4 "FIRST 24 RECORDS IN OUTPUT DATA SET &OUTFILE_MEAS.";
run;

***----- TO PRINT VARIABLE LABELS COMMENT (DELETE) "NOLABELS" FROM PROC MEANS STATEMENTS -------***;

proc Means data = OUTMSR.&OUTFILE_MEAS. n nmiss min max nolabels ;
      var KEY HOSPID DRG DRGVER MDC MDRG YEAR DQTR 
         AGECAT POPCAT  SEXCAT RACECAT PAYCAT DUALCAT LOS TRNSFER;
     title4 "CATEGORICAL VARIABLES AND RANGES OF VALUES";
run; quit;

proc Means data = OUTMSR.&OUTFILE_MEAS. n nmiss sum mean nolabels ;
      var CHF VALVE PULMCIRC PERIVASC HTN_C  PARA NEURO CHRNLUNG DM DMCX
         HYPOTHY RENLFAIL LIVER ULCER AIDS  LYMPH METS TUMOR ARTH OBESE
         WGHTLOSS BLDLOSS ANEMDEF ALCOHOL DRUG PSYCH DEPRESS;
     title "PATIENT SAFETY INDICATOR CASES CO-MORBIDITY COUNTS (=SUM) AND RATES (=MEAN)";
run; quit;

proc Means data = OUTMSR.&OUTFILE_MEAS. n nmiss sum nolabels ;
     var TAPS21-TAPS27;
    * OUTPUT OUT=STATS1;
     title "PATIENT SAFETY INDICATOR NUMERATORS FOR AREA-BASED OR COUNT-BASED INDICATORS (COUNT =SUM)";
run; quit;

proc Means data = OUTMSR.&OUTFILE_MEAS. n nmiss sum mean nolabels ;
     var TPPS02-TPPS04  TPPS04A--TPPS04E TPPS05-TPPS16 TPPS18 TPPS19;
    * OUTPUT OUT=STATS3;
    title1 "PATIENT SAFETY PROVIDER LEVEL INDICATOR NUMERATORS (=SUM),DENOMINATORS (=N), AND OBSERVED RATE (MEAN) ";
    title2 "WITH POA MODIFIERS";
run; quit;

